"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addSearchParams = exports.addPaths = void 0;
const ErrorCode_1 = require("../error/ErrorCode");
const OpenIdFederationError_1 = require("../error/OpenIdFederationError");
/**
 *
 * Add paths to a url
 *
 * ## example
 *
 * ```typescript
 * const url = addPaths('https://example.org','path', 'two')
 * assert(url, "https://example.org/path/two")
 *
 * const url = addPaths('https://example.org/','path', 'two')
 * assert(url, "https://example.org/path/two")
 *
 * const url = addPaths('https://example.org/','path/', '/two/')
 * assert(url, "https://example.org/path/two")
 * ```
 *
 */
const addPaths = (baseUrl, ...paths) => {
    const [scheme, rest] = baseUrl.split('://');
    if (!rest) {
        throw new OpenIdFederationError_1.OpenIdFederationError(ErrorCode_1.ErrorCode.Validation, 'not a valid URL');
    }
    const urlWithoutScheme = rest
        // Get all base the parts
        .split('/')
        // Add the paths
        .concat(...paths.map((p) => p.split('/')))
        // Filter out empty paths (i.e. '///path///' to 'path')
        .filter((s) => s.length > 0)
        // Create the full url
        .join('/');
    return `${scheme}://${urlWithoutScheme}`;
};
exports.addPaths = addPaths;
const addSearchParams = (baseUrl, searchParams) => baseUrl.endsWith('?')
    ? `${baseUrl}${searchParams instanceof URLSearchParams ? searchParams : new URLSearchParams(searchParams)}`
    : `${baseUrl}?${searchParams instanceof URLSearchParams ? searchParams : new URLSearchParams(searchParams)}`;
exports.addSearchParams = addSearchParams;
//# sourceMappingURL=url.js.map